@@ -1,7 +1,7 @@
 /*
  *  backup.h
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,11 +21,13 @@
 #define _ALPM_BACKUP_H
 
 #include "alpm_list.h"
+#include "alpm.h"
 
-char *_alpm_backup_file(const char *string);
-char *_alpm_backup_hash(const char *string);
-char *_alpm_needbackup(const char *file, const alpm_list_t *backup);
+int _alpm_split_backup(const char *string, alpm_backup_t **backup);
+alpm_backup_t *_alpm_needbackup(const char *file, alpm_pkg_t *pkg);
+void _alpm_backup_free(alpm_backup_t *backup);
+alpm_backup_t *_alpm_backup_dup(const alpm_backup_t *backup);
 
 #endif /* _ALPM_BACKUP_H */
 
-/* vim: set ts=2 sw=2 noet: */
+/* vim: set noet: */
