@@ -1,7 +1,7 @@
 /*
  *  delta.h
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2007-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -24,29 +24,12 @@
 
 #include "alpm.h"
 
-struct __pmdelta_t {
-	/** filename of the delta patch */
-	char *delta;
-	/** md5sum of the delta file */
-	char *delta_md5;
-	/** filesize of the delta file */
-	off_t delta_size;
-	/** filename of the 'before' file */
-	char *from;
-	/** filename of the 'after' file */
-	char *to;
-	/** download filesize of the delta file */
-	off_t download_size;
-};
-
-pmdelta_t *_alpm_delta_parse(char *line);
-void _alpm_delta_free(pmdelta_t *delta);
-off_t _alpm_shortest_delta_path(alpm_list_t *deltas,
+alpm_delta_t *_alpm_delta_parse(alpm_handle_t *handle, const char *line);
+void _alpm_delta_free(alpm_delta_t *delta);
+alpm_delta_t *_alpm_delta_dup(const alpm_delta_t *delta);
+off_t _alpm_shortest_delta_path(alpm_handle_t *handle, alpm_list_t *deltas,
 		const char *to, alpm_list_t **path);
 
-/* max percent of package size to download deltas */
-#define MAX_DELTA_RATIO 0.7
-
 #endif /* _ALPM_DELTA_H */
 
-/* vim: set ts=2 sw=2 noet: */
+/* vim: set noet: */
