@@ -1,7 +1,7 @@
 /*
  *  group.c
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,8 +18,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <stdlib.h>
 #include <string.h>
 
@@ -32,7 +30,7 @@
 
 alpm_group_t *_alpm_group_new(const char *name)
 {
-	alpm_group_t* grp;
+	alpm_group_t *grp;
 
 	CALLOC(grp, 1, sizeof(alpm_group_t), return NULL);
 	STRDUP(grp->name, name, free(grp); return NULL);
@@ -52,4 +50,4 @@ void _alpm_group_free(alpm_group_t *grp)
 	FREE(grp);
 }
 
-/* vim: set ts=2 sw=2 noet: */
+/* vim: set noet: */
