@@ -1,7 +1,7 @@
 /*
  *  database.c
  *
- *  Copyright (c) 2006-2011 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,8 +18,6 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "config.h"
-
 #include <stdio.h>
 
 #include <alpm.h>
@@ -63,11 +61,11 @@ int pacman_database(alpm_list_t *targets)
 		return 1;
 	}
 
-	db_local = alpm_option_get_localdb(config->handle);
+	db_local = alpm_get_localdb(config->handle);
 	for(i = targets; i; i = alpm_list_next(i)) {
 		char *pkgname = i->data;
 		alpm_pkg_t *pkg = alpm_db_get_pkg(db_local, pkgname);
-		if(!pkg || alpm_db_set_pkgreason(config->handle, pkg, reason)) {
+		if(!pkg || alpm_pkg_set_reason(pkg, reason)) {
 			pm_printf(ALPM_LOG_ERROR, _("could not set install reason for package %s (%s)\n"),
 							pkgname, alpm_strerror(alpm_errno(config->handle)));
 			retval = 1;
@@ -87,4 +85,4 @@ int pacman_database(alpm_list_t *targets)
 	return retval;
 }
 
-/* vim: set ts=2 sw=2 noet: */
+/* vim: set noet: */
