@@ -1,8 +1,9 @@
-#! /usr/bin/python
+#! /usr/bin/python2
 #
 #  pactest : run automated testing on the pacman binary
 #
 #  Copyright (c) 2006 by Aurelien Foret <orelien@chez.com>
+#  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -17,7 +18,6 @@
 #  You should have received a copy of the GNU General Public License
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import glob
 from optparse import OptionParser
 import os
 import shutil
@@ -25,29 +25,14 @@ import sys
 import tempfile
 
 import pmenv
+import tap
 import util
 
 __author__ = "Aurelien FORET"
 __version__ = "0.4"
 
-def resolve_binary_path(option, opt_str, value, parser):
-    setattr(parser.values, option.dest, os.path.abspath(value))
-
-def glob_tests(option, opt_str, value, parser):
-    idx = 0
-    globlist = []
-
-    # maintain the idx so we can modify rargs
-    while idx < len(parser.rargs) and \
-            not parser.rargs[idx].startswith('-'):
-        globlist += glob.glob(parser.rargs[idx])
-        idx += 1
-
-    parser.rargs = parser.rargs[idx:]
-    setattr(parser.values, option.dest, globlist)
-
 def create_parser():
-    usage = "usage: %prog [options] [[--test <path/to/testfile.py>] ...]"
+    usage = "usage: %prog [options] <path/to/testfile.py>..."
     description = "Runs automated tests on the pacman binary. Tests are " \
             "described using an easy python syntax, and several can be " \
             "ran at once."
@@ -59,13 +44,12 @@ def create_parser():
     parser.add_option("-d", "--debug", type = "int",
                       dest = "debug", default = 0,
                       help = "set debug level for pacman")
-    parser.add_option("-p", "--pacman", action = "callback",
-                      callback = resolve_binary_path, type = "string",
-                      dest = "bin", default = "pacman",
+    parser.add_option("-p", "--pacman", type = "string",
+                      dest = "bin", default = None,
                       help = "specify location of the pacman binary")
-    parser.add_option("-t", "--test", action = "callback",
-                      callback = glob_tests, dest = "testcases",
-                      help = "specify test case(s)")
+    parser.add_option("--bindir", type = "string",
+                      dest = "bindir", action = "append",
+                      help = "specify location of binaries")
     parser.add_option("--keep-root", action = "store_true",
                       dest = "keeproot", default = False,
                       help = "don't remove the generated pacman root filesystem")
@@ -81,41 +65,61 @@ def create_parser():
     parser.add_option("--manual-confirm", action = "store_true",
                       dest = "manualconfirm", default = False,
                       help = "do not use --noconfirm for pacman calls")
+    parser.add_option("--scriptlet-shell", type = "string",
+                      dest = "scriptletshell", default = "/bin/sh",
+                      help = "specify path to shell used for install scriptlets")
+    parser.add_option("--ldconfig", type = "string",
+                      dest = "ldconfig", default = "/sbin/ldconfig",
+                      help = "specify path to ldconfig")
     return parser
 
 
 if __name__ == "__main__":
-    # instantiate env and parser objects
-    root_path = tempfile.mkdtemp()
-    env = pmenv.pmenv(root=root_path)
+
+    if sys.hexversion < 0x02070000:
+        # bailing now with clear message better than mid-run with unhelpful one
+        tap.bail("Python versions before 2.7 are not supported.")
+        sys.exit(1)
+
+    # parse options
     opt_parser = create_parser()
     (opts, args) = opt_parser.parse_args()
 
+    if args is None or len(args) == 0:
+        tap.bail("no tests defined, nothing to do")
+        sys.exit(2)
+
+    # instantiate env
+    root_path = tempfile.mkdtemp(prefix='pactest-')
+    env = pmenv.pmenv(root=root_path)
+
     # add parsed options to env object
     util.verbose = opts.verbose
     env.pacman["debug"] = opts.debug
     env.pacman["bin"] = opts.bin
+    env.pacman["bindir"] = opts.bindir
     env.pacman["nolog"] = opts.nolog
     env.pacman["gdb"] = opts.gdb
     env.pacman["valgrind"] = opts.valgrind
     env.pacman["manual-confirm"] = opts.manualconfirm
+    env.pacman["scriptlet-shell"] = opts.scriptletshell
+    env.pacman["ldconfig"] = opts.ldconfig
 
-    if opts.testcases is None or len(opts.testcases) == 0:
-        print "no tests defined, nothing to do"
+    try:
+        for i in args:
+            env.addtest(i)
+    except Exception as e:
+        tap.bail(e)
         os.rmdir(root_path)
         sys.exit(2)
 
-    for i in opts.testcases:
-        env.addtest(i)
-
-    # run tests and print overall results
+    # run tests
     env.run()
-    env.results()
 
     if not opts.keeproot:
         shutil.rmtree(root_path)
     else:
-        print "pacman testing root saved: %s" % root_path
+        tap.diag("pacman testing root saved: %s" % root_path)
 
     if env.failed > 0:
         sys.exit(1)
