@@ -1,6 +1,5 @@
-#! /usr/bin/python
-#
 #  Copyright (c) 2006 by Aurelien Foret <orelien@chez.com>
+#  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -15,11 +14,7 @@
 #  You should have received a copy of the GNU General Public License
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
 import os
-import tempfile
-import stat
-import shutil
 from StringIO import StringIO
 import tarfile
 
@@ -68,6 +63,7 @@ class pmpkg(object):
             "post_upgrade": "",
         }
         self.path = None
+        self.finalized = False
 
     def __str__(self):
         s = ["%s" % self.fullname()]
@@ -158,6 +154,8 @@ class pmpkg(object):
             info = tarfile.TarInfo(fileinfo["filename"])
             if fileinfo["hasperms"]:
                 info.mode = fileinfo["perms"]
+            elif fileinfo["isdir"]:
+                info.mode = 0o755
             if fileinfo["isdir"]:
                 info.type = tarfile.DIRTYPE
                 tar.addfile(info)
@@ -181,34 +179,45 @@ class pmpkg(object):
             if os.path.isfile(path):
                 os.utime(path, (355, 355))
 
-    def full_filelist(self):
-        """Generate a list of package files.
+    def filelist(self):
+        """Generate a list of package files."""
+        return sorted([self.parse_filename(f) for f in self.files])
 
-        Each path is decomposed to generate the list of all directories leading
-        to the file.
+    def finalize(self):
+        """Perform any necessary operations to ready the package for use."""
+        if self.finalized:
+            return
 
-        Example with 'usr/local/bin/dummy':
-        The resulting list will be:
-            usr/
-            usr/local/
-            usr/local/bin/
-            usr/local/bin/dummy
-        """
-        file_set = set()
-        for name in self.files:
-            name = self.parse_filename(name)
-            file_set.add(name)
-            while "/" in name:
-                name, tmp = name.rsplit("/", 1)
-                file_set.add(name + "/")
-        return sorted(file_set)
+        # add missing parent dirs to file list
+        # use bare file names so trailing ' -> ', '*', etc don't throw off the
+        # checks for existing files
+        file_names = self.filelist()
+        for name in list(file_names):
+            if os.path.isabs(name):
+                raise ValueError("Absolute path in filelist '%s'." % name)
+
+            name = os.path.dirname(name.rstrip("/"))
+            while name:
+                if name in file_names:
+                    # path exists as both a file and a directory
+                    raise ValueError("Duplicate path in filelist '%s'." % name)
+                elif name + "/" in file_names:
+                    # path was either manually included or already processed
+                    break
+                else:
+                    file_names.append(name + "/")
+                    self.files.append(name + "/")
+                name = os.path.dirname(name)
+        self.files.sort()
+
+        self.finalized = True
 
     def local_backup_entries(self):
         return ["%s\t%s" % (self.parse_filename(i), util.mkmd5sum(i)) for i in self.backup]
 
     def installfile(self):
         data = []
-        for key, value in self.install.iteritems():
+        for key, value in self.install.items():
             if value:
                 data.append("%s() {\n%s\n}\n" % (key, value))
 
