@@ -1,7 +1,5 @@
-#! /usr/bin/python2
-#
 #  Copyright (c) 2006 by Aurelien Foret <orelien@chez.com>
-#  Copyright (c) 2006-2013 Pacman Development Team <pacman-dev@archlinux.org>
+#  Copyright (c) 2006-2014 Pacman Development Team <pacman-dev@archlinux.org>
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,6 +17,7 @@
 import os
 import stat
 
+import tap
 import util
 
 class pmrule(object):
@@ -31,9 +30,13 @@ class pmrule(object):
         self.result = 0
 
     def __str__(self):
-        if len(self.rule) <= 40:
-            return self.rule
-        return self.rule[:37] + '...'
+        return self.rule
+
+    def snapshots_needed(self):
+        (testname, args) = self.rule.split("=")
+        if testname == "FILE_MODIFIED" or testname == "!FILE_MODIFIED":
+            return [args]
+        return []
 
     def check(self, test):
         """
@@ -57,12 +60,12 @@ class pmrule(object):
             elif case == "OUTPUT":
                 logfile = os.path.join(test.root, util.LOGFILE)
                 if not os.access(logfile, os.F_OK):
-                    print "LOGFILE not found, cannot validate 'OUTPUT' rule"
+                    tap.diag("LOGFILE not found, cannot validate 'OUTPUT' rule")
                     success = 0
                 elif not util.grep(logfile, key):
                     success = 0
             else:
-                print "PACMAN rule '%s' not found" % case
+                tap.diag("PACMAN rule '%s' not found" % case)
                 success = -1
         elif kind == "PKG":
             localdb = test.db["local"]
@@ -100,21 +103,23 @@ class pmrule(object):
                     if not value in newpkg.files:
                         success = 0
                 elif case == "BACKUP":
-                    found = 0
+                    success = 0
                     for f in newpkg.backup:
-                        name, md5sum = f.split("\t")
-                        if value == name:
-                            found = 1
-                    if not found:
-                        success = 0
+                        if f.startswith(value + "\t"):
+                            success = 1
+                            break;
                 else:
-                    print "PKG rule '%s' not found" % case
+                    tap.diag("PKG rule '%s' not found" % case)
                     success = -1
         elif kind == "FILE":
             filename = os.path.join(test.root, key)
             if case == "EXIST":
                 if not os.path.isfile(filename):
                     success = 0
+            elif case == "EMPTY":
+                if not (os.path.isfile(filename)
+                        and os.path.getsize(filename) == 0):
+                    success = 0
             elif case == "MODIFIED":
                 for f in test.files:
                     if f.name == key:
@@ -148,7 +153,7 @@ class pmrule(object):
                 if not os.path.isfile("%s.pacsave" % filename):
                     success = 0
             else:
-                print "FILE rule '%s' not found" % case
+                tap.diag("FILE rule '%s' not found" % case)
                 success = -1
         elif kind == "DIR":
             filename = os.path.join(test.root, key)
@@ -156,7 +161,7 @@ class pmrule(object):
                 if not os.path.isdir(filename):
                     success = 0
             else:
-                print "DIR rule '%s' not found" % case
+                tap.diag("DIR rule '%s' not found" % case)
                 success = -1
         elif kind == "LINK":
             filename = os.path.join(test.root, key)
@@ -164,7 +169,7 @@ class pmrule(object):
                 if not os.path.islink(filename):
                     success = 0
             else:
-                print "LINK rule '%s' not found" % case
+                tap.diag("LINK rule '%s' not found" % case)
                 success = -1
         elif kind == "CACHE":
             cachedir = os.path.join(test.root, util.PM_CACHEDIR)
@@ -174,7 +179,7 @@ class pmrule(object):
                         os.path.join(cachedir, pkg.filename())):
                     success = 0
         else:
-            print "Rule kind '%s' not found" % kind
+            tap.diag("Rule kind '%s' not found" % kind)
             success = -1
 
         if self.false and success != -1:
